var TeslaCloud = window.TeslaCloud || {};
TeslaCloud.Pluralization = window.TeslaCloud.Pluralization || {};

!function ($, window, document, _undefined)
{
    "use strict";

    TeslaCloud.Pluralization = XF.create({
        language: null,

        pluralRules: null,

        __construct: function (language)
        {
            this.language = language = language || $('html').attr('lang');
            if (!TeslaCloud.Pluralization._isSupported)
            {
                console.error('Pluralization is not supported');

                return;
            }

            try
            {
                this.pluralRules = new Intl.PluralRules(language);
            }
            catch (e)
            {
                console.error(`Pluralization init error:`, e)
            }
        },

        pluralPhraseName: function (number, phrases)
        {
            if (typeof phrases == 'string')
            {
                phrases = {
                    'zero': phrases + '_zero',
                    'one': phrases + '_one',
                    'two': phrases + '_two',
                    'few': phrases + '_few',
                    'many': phrases + '_many',
                    'other': phrases
                };
            }

            if (!phrases.hasOwnProperty('other'))
            {
                console.error('Required pluralization form "other" is missing');

                return '';
            }

            return phrases[this.getCardinal(number, Object.keys(phrases))];
        },

        phrasePlural: function (number, phrases, vars, fallback)
        {
            return XF.phrase(this.pluralPhraseName(number, phrases), vars, fallback)
        },

        getCardinal: function (number, availableCardinals = [])
        {
            if (!TeslaCloud.Pluralization._isSupported || !this.pluralRules)
            {
                return 'other';
            }

            const cardinal = this.pluralRules.select(number);
            if (!availableCardinals.length || availableCardinals.includes(cardinal))
            {
                return cardinal;
            }

            return 'other';
        }
    });
    TeslaCloud.Pluralization._isSupported = 'Intl' in window && 'PluralRules' in window.Intl;
    TeslaCloud.Pluralization.current = new TeslaCloud.Pluralization;

    const XFPhrase = XF['phrase'];
    $.extend(XF, {
        phrase: function (name, vars, fallback)
        {
            const pluralHelperRegExp = /\{plural\s+((?:(?:[a-z]|[0-9.0-9])+=(['"]).+?\2\s*)+)}/g,
                phraseText = XFPhrase(name, vars, fallback);

            if (!pluralHelperRegExp.test(phraseText))
            {
                return phraseText;
            }

            if (!TeslaCloud.Pluralization._isSupported && fallback)
            {
                return fallback;
            }

            return phraseText.replaceAll(pluralHelperRegExp, function (match, argumentString, quote)
            {
                const argMatches = argumentString.matchAll(
                        new RegExp(`(.+?)=${quote}([^${quote}]*)${quote}\\s*`, 'g')
                    ),
                    args = {};

                for (const match of argMatches)
                {
                    args[match[1]] = match[2];
                }

                if (!args.hasOwnProperty('number'))
                {
                    console.error(`Phrase pluralization error: ${name}: Missing parameter "number"`);

                    return '';
                }

                if (!args.hasOwnProperty('other'))
                {
                    console.error(`Phrase pluralization error: ${name}: Missing parameter "other"`);

                    return '';
                }

                const pluralization = TeslaCloud.Pluralization.current,
                    number = args['number'],
                    precision = args['precision'] || 0,
                    formattedNumber = Number(number).toLocaleString(pluralization.language, {
                        minimumFractionDigits: precision,
                        maximumFractionDigits: precision
                    });
                if (args.hasOwnProperty(number))
                {
                    return args[number]
                        .replace('#n', formattedNumber);
                }

                return args[pluralization.getCardinal(number, Object.keys(args))]
                    .replace('#n', formattedNumber)
            });
        }
    });
}
(jQuery, window, document);